package ch1.sec3;

import java.util.Iterator;

public class LinkDeque<Item> implements Iterable<Item> {
    private Node left;
    private Node right;
    private int size;

    @Override
    public LinkDequeIterator iterator() {
        return new LinkDequeIterator();
    }

    public LinkDeque() {
        left = null;
        right = null;
        size = 0;
    }

    public boolean isEmpty() {
        return size == 0;
    }

    public int size() {
        return size;
    }

    public void pushLeft(Item item) {
        left = new Node(item, null, left);
        if (isEmpty()) {
            right = left;
        } else {
            left.next.prev = left;
        }
        size++;
    }

    public void pushRight(Item item) {
        right = new Node(item, right, null);
        if (isEmpty()) {
            left = right;
        } else {
            right.prev.next = right;
        }
        size++;
    }

    public Item popLeft() {
        if (isEmpty()) {
            return null;
        }
        Item item = left.value;
        left = left.next;
        if (size() == 1) {
            right = null;
        } else {
            left.prev = null;
        }
        size--;
        return item;
    }

    public Item popRight() {
        if (isEmpty()) {
            return null;
        }
        Item item = right.value;
        right = right.prev;
        if (size() == 1) {
            left = null;
        } else {
            right.next = null;
        }
        size--;
        return item;
    }

    public static void main(String[] args) {
        LinkDeque<Integer> linkDeque = new LinkDeque<>();
        System.out.println(linkDeque.isEmpty());
        linkDeque.pushLeft(1);
        linkDeque.pushLeft(2);
        linkDeque.pushRight(3);
        linkDeque.pushRight(4);
        for (Integer i : linkDeque) {
            System.out.println(i);
        }
        System.out.println(linkDeque.size());
        while (!linkDeque.isEmpty()) {
            if (linkDeque.size() % 2 == 0) {
                System.out.println(linkDeque.popLeft());
            } else {
                System.out.println(linkDeque.popRight());
            }
        }
    }

    private class Node {
        private Item value;
        private Node prev;
        private Node next;
        public Node(Item value, Node prev, Node next) {
            this.value = value;
            this.prev = prev;
            this.next = next;
        }
    }

    private class LinkDequeIterator implements Iterator<Item> {
        private Node current;

        public LinkDequeIterator() {
            current = left;
        }

        @Override
        public boolean hasNext() {
            return current != null;
        }

        @Override
        public Item next() {
            Item item = current.value;
            current = current.next;
            return item;
        }
    }
}
