package ch1.sec3;

import java.util.Iterator;

public class Queue<Item> implements Iterable<Item> {
    private Item[] items;
    private int size;
    private int head;
    private int tail;

    @Override
    public QueueIterator iterator() {
        return new QueueIterator();
    }

    public Queue() {
        items = (Item[]) new Object[4];
        size = 0;
        head = 0;
        tail = 0;
    }

    public void enqueue(Item item) {
        if (size >= items.length) {
            resize(items.length * 2);
        }
        if (!isEmpty()) {
            tail = (tail + 1) % items.length;
        }
        items[tail] = item;
        size++;
    }

    public Item dequeue() {
        if (isEmpty()) {
            return null;
        }
        if (size <= items.length / 4) {
            resize(items.length / 2);
        }
        Item item = items[head];
        items[head] = null;
        if (size() != 1) {
            head = (head + 1) % items.length;
        }
        size--;
        return item;
    }

    public boolean isEmpty() {
        return size == 0;
    }

    public int size() {
        return size;
    }

    public static void main(String[] args) {
        Queue<Integer> queue= new Queue<>();
        System.out.println(queue.isEmpty());
        queue.enqueue(1);
        queue.enqueue(2);
        queue.enqueue(3);
        queue.enqueue(4);
        queue.enqueue(5);
        for (Integer i : queue) {
            System.out.println(i);
        }
        System.out.println(queue.size());
        while (!queue.isEmpty()) {
            System.out.println(queue.dequeue());
        }
    }

    private class QueueIterator implements Iterator<Item> {
        private int index;

        QueueIterator() {
            index = 0;
        }

        @Override
        public boolean hasNext() {
            return index < size;
        }

        @Override
        public Item next() {
            int nextIndex = (head + index) % items.length;
            index++;
            return items[nextIndex];
        }
    }

    private void resize(int newsize) {
        Item[] newItems = (Item[]) new Object[newsize];
        for(int i = 0; i < size; i++) {
            newItems[i] = items[(head + i) % items.length];
        }
        head = 0;
        tail = size == 0 ? 0 : size - 1;
        items = newItems;
    }
}