package ch1.sec3;

import java.util.Iterator;

public class Stack<Item> implements Iterable<Item> {
    private Item[] items;
    private int size;

    @Override
    public StackIterator iterator() {
        return new StackIterator();
    }

    public Stack() {
        items = (Item[]) new Object[4];
        size = 0;
    }

    public void push(Item item) {
        if (size >= items.length) {
            resize(items.length * 2);
        }
        items[size++] = item;
    }

    public Item pop() {
        if (isEmpty()) {
            return null;
        }
        if (size <= items.length / 4) {
            resize(items.length / 2);
        }
        Item item = items[--size];
        items[size] = null;
        return item;
    }

    public boolean isEmpty() {
        return size == 0;
    }

    public int size() {
        return size;
    }

    public static void main(String[] args) {
        Stack<Integer> stack = new Stack<>();
        System.out.println(stack.isEmpty());
        stack.push(1);
        stack.push(2);
        stack.push(3);
        stack.push(4);
        stack.push(5);
        for (Integer i : stack) {
            System.out.println(i);
        }
        System.out.println(stack.size());
        while (!stack.isEmpty()) {
            System.out.println(stack.pop());
        }
    }

    private class StackIterator implements Iterator<Item> {
        private int index;

        StackIterator() {
            index = size;
        }

        @Override
        public boolean hasNext() {
            return index != 0;
        }

        @Override
        public Item next() {
            return items[--index];
        }
    }

    private void resize(int newSize) {
        Item[] newItems = (Item[]) new Object[newSize];
        for(int i = 0; i < size; i++) {
            newItems[i] = items[i];
        }
        items = newItems;
    }
}
